@@ -1,4 +1,4 @@
-from typing import Dict, List, Optional
+from typing import Dict, List, Optional, Type
 
 from .ambient import Ambient
 from .physics.action_executor import ActionExecutor
@@ -9,10 +9,10 @@ from ..common.message import Message, BccMessage
 from ..common.action import Action
 from ..common.perception import Perception
 from ..common.action_result import ActionResult
+from ..common.exceptions import IdentityException
 from ..utils.utils import ignore
 
 
-
 class Environment():
     def __init__(self, ambient: Ambient, initial_actors: List[Actor]=[], initial_passive_bodies: List[Body]=[]) -> None:
         self.__ambient: Ambient = ambient
@@ -66,20 +66,27 @@ class Environment():
 
         del self.__passive_bodies[passive_body_id]
 
-    def generate_perception_for_actor(self, actor_id: str, action_result: ActionResult) -> Optional[Perception]:
+    def generate_perception_for_actor(self, actor_id: str, action_type: Type[Action], action_result: ActionResult) -> Optional[Perception]:
         # Abstract.
         ignore(self)
         ignore(actor_id)
+        ignore(action_type)
         ignore(action_result)
 
         return None
 
-    def send_message_to_actors(self, message: Message) -> None:
+    def send_message_to_recipients(self, message: Message, check_sender_identity: bool=True) -> None:
+        if check_sender_identity and message.get_sender_id() not in self.__actors:
+            raise IdentityException("Unknown sender: {}. The sender must be an actor living in the environment.".format(message.get_sender_id()))
+
         if message.get_recipients_ids() == []:
             message.override_recipients(recipient_ids=[recipient_id for recipient_id in self.__actors if recipient_id != message.get_sender_id()])
 
+        self.__send_message_to_recipients(message=message)
+
+    def __send_message_to_recipients(self, message: Message) -> None:
         for recipient_id in message.get_recipients_ids():
-            if recipient_id in self.__actors:
+            if recipient_id in self.__actors and recipient_id != message.get_sender_id():
                 bcc_message: BccMessage = BccMessage(content=message.get_content(), sender_id=message.get_sender_id(), recipient_id=recipient_id)
                 recipient_listening_sensor: Sensor = self.__actors[recipient_id].get_listening_sensor()
 
@@ -100,7 +107,7 @@ class Environment():
 
     def __execute_actor_actions(self, actor: Actor) -> None:
         actor.cycle()
-        actions: List[Action] = actor.get_outstanding_actions()
+        actions: List[Action] = actor.get_pending_actions()
         self.validate_actions(actions=actions)
 
         for action in actions:
@@ -108,21 +115,20 @@ class Environment():
 
     def validate_actions(self, actions: List[Action]) -> None:
         # Abstract.
-        
+
         ignore(self)
         ignore(actions)
 
     def execute_action(self, action: Action) -> None:
-        action_executor: ActionExecutor = self.get_executor_for(action=action)
+        action_executor: Optional[ActionExecutor] = self.get_executor_for(action=action)
 
         if not action_executor:
             raise ValueError("No executor found for action of type {}.".format(type(action)))
         else:
             result: ActionResult = action_executor.execute(env=self, action=action)
-            perception: Perception = self.generate_perception_for_actor(actor_id=action.get_actor_id(), action_result=result)
-            
-            self.send_perception_to_actor(perception=perception, actor_id=action.get_actor_id())
+            perception: Perception = self.generate_perception_for_actor(action_type=type(action), actor_id=action.get_actor_id(), action_result=result)
 
+            self.send_perception_to_actor(perception=perception, actor_id=action.get_actor_id())
 
     def get_executor_for(self, action: Action) -> Optional[ActionExecutor]:
         # Abstract.
